@@ -12,7 +12,6 @@
 #   calculateCosPhi: to calculate components' cos(phi(t)).
 ################################################################################
 
-
 ################################################################################
 # Internal function: Returns the RSS for an estimation given (alpha,omega).
 # Arguments:
@@ -47,7 +46,7 @@ step1FMM <- function(optBase, vData) {
 ################################################################################
 
 step2FMM <- function(parameters, vData, timePoints, omegaMin, omegaMax){
-  if(parameters[2] > omegaMin  &  parameters[2] < omegaMax){
+  if(parameters[2] >= omegaMin  &  parameters[2] <= omegaMax){
     # Linear parameters estimation
     nonlinearMob = 2*atan(parameters[2]*tan((timePoints-parameters[1])/2))
     DM <- cbind(rep(1, length(timePoints)), cos(nonlinearMob), sin(nonlinearMob))
@@ -80,12 +79,12 @@ PV <- function(vData, pred){
 ################################################################################
 PVj <- function(vData, timePoints, alpha, beta, omega){
   # Fitted values of each wave
-  nComponents <- length(alpha)
-  waves <- calculateCosPhi(alpha = alpha, beta = beta, omega = omega,
-                           timePoints = timePoints)
+  DM <- calculateDesignMatrix(alpha = alpha, omega = omega, timePoints = timePoints)
 
   # The percentage of variability explained up to wave i is determined
-  cumulativePV <- sapply(1:nComponents, function(x){PV(vData, predict(lm(vData ~ waves[,1:x])))})
+  cumulativePV <- sapply(1:length(alpha), function(x){
+    PV(vData, predict(lm(vData ~ DM[,1:(2*x+1)]-1)))
+  })
 
   # individual percentage of variability is the part that adds to the whole
   return(c(cumulativePV[1], diff(cumulativePV)))
@@ -101,6 +100,17 @@ seqTimes <- function(nObs){
   return(seq(0, 2*pi, length.out = nObs+1)[1:nObs])
 }
 
+################################################################################
+# Internal function: to calculate a single components' cos(phi(t)).
+# Arguments:
+#   alpha, beta, omega: parameters.
+#   timePoints: time points in which the FMM model is computed.
+# Returns a matrix of each component's cos(phi(t)) as columns.
+################################################################################
+calculateSingleCosPhi <- function(alpha, beta, omega, timePoints){
+  return(cos(beta + 2*atan(omega*tan((timePoints - alpha)/2))))
+}
+
 ################################################################################
 # Internal function: to calculate components' cos(phi(t)).
 # Arguments:
@@ -109,10 +119,30 @@ seqTimes <- function(nObs){
 # Returns a matrix of each component's cos(phi(t)) as columns.
 ################################################################################
 calculateCosPhi <- function(alpha, beta, omega, timePoints){
-  calculateSingleCosPhi <- function(alpha, beta, omega){
+  calculateSingleCosPhi <- function(alpha, beta, omega, timePoints){
     return(cos(beta + 2*atan(omega*tan((timePoints - alpha)/2))))
   }
-  return(mapply(FUN = calculateSingleCosPhi, alpha = alpha, beta = beta, omega = omega))
+  return(sapply(1:length(alpha), FUN = function(x){
+    calculateSingleCosPhi(alpha = alpha[x], beta = beta[x], omega = omega[x], timePoints = timePoints)
+  }))
+  # return(mapply(FUN = calculateSingleCosPhi, alpha = alpha, beta = beta, omega = omega, timePoints=timePoints))
+}
+
+################################################################################
+# Internal function: to calculate design matrix with fixed alphas and omegas
+# Arguments:
+#   alpha, omega: parameters.
+#   timePoints: time points in which the FMM model is computed.
+# Returns the design matrix (fixed alphas and omegas)
+################################################################################
+calculateDesignMatrix <- function(alpha, omega, timePoints){
+  design_matrix <- matrix(1, nrow = length(timePoints), ncol = 1)
+  for (i in seq_along(alpha)) {
+    cos_col <- calculateCosPhi(alpha[i], 0, omega[i], timePoints)
+    sin_col <- calculateCosPhi(alpha[i], -pi/2, omega[i], timePoints) # sin(f) = cos(f-pi/2)
+    design_matrix <- cbind(design_matrix, cos_col, sin_col)
+  }
+  return(design_matrix)
 }
 
 ################################################################################
@@ -131,23 +161,36 @@ precalculateBase <- function(alphaGrid, omegaGrid, timePoints){
   grid <- expand.grid(alphaGrid, omegaGrid)
   optBase <- apply(grid, 1, FUN = function(x){
     x <- as.numeric(x)
-    nonlinearMob = 2*atan(x[2]*tan((timePoints-x[1])/2))
-    M <- cbind(timePoints*0+1, cos(nonlinearMob), sin(nonlinearMob))
-    mat <- t(M) %*% M
+    DM <- calculateDesignMatrix(x[1], x[2], timePoints)
+    mat <- t(DM) %*% DM
+
     # Check if matrix is invertible, too low omegas and extreme timepoints
     # configurations lead to computationally singular systems
     if (det(mat) < 10e-12){
       return(NULL)
     }else{
-      return(list(base = solve(mat)%*%t(M),
+      return(list(base = solve(mat)%*%t(DM),
                   alpha = x[1], omega = x[2],
-                  cost = cos(nonlinearMob), sint = sin(nonlinearMob)))
+                  cost = cos(2*atan(x[2]*tan((timePoints-x[1])/2))),
+                  sint = sin(2*atan(x[2]*tan((timePoints-x[1])/2)))))
     }
   }, simplify = FALSE)
   optBase <- Filter(Negate(is.null), optBase)
   return(optBase)
 }
 
+################################################################################
+# Internal function: to calculate the Mobius transform in e^it with argument a
+# Arguments:
+#   a: parameter.
+#   timePoints: time points in which the Mobius transform is computed.
+# Returns the transform
+################################################################################
+mobius <- function(a, t){
+  return((exp(1i*t)-a) / (1-Conj(a)*exp(1i*t)))
+}
+
+
 ################################################################################
 # Internal function: to check if the arguments passed to fitFMM are correct
 # Arguments: See fitFMM function arguments
@@ -197,23 +240,32 @@ checkArguments <- function(vData, nPeriods, timePoints, nback, maxiter,
 checkSolution <- function(fittedFMM, omegaMin, omegaMax){
 
   # Omegas
-  if(any(getOmega(fittedFMM) == omegaMin)) warning("Detected unusually small omegas. Check your model parameters and input data.")
+  if(any(getOmega(fittedFMM) <= omegaMin)) warning("Detected unusually small omegas. Check your model parameters and input data.")
 
   # Amplitudes
   if(any(getA(fittedFMM) < 0)) stop("Invalid solution: check function input parameters.")
 
-  residuals <- fittedFMM@data
-  amplitudeFlag <- FALSE
-  for (i in 1:length(fittedFMM@alpha)) {
-    nonlinearMob = 2*atan(fittedFMM@omega[i]*tan((fittedFMM@timePoints-fittedFMM@alpha[i])/2))
-    DM <- cbind(fittedFMM@timePoints*0+1, cos(nonlinearMob), sin(nonlinearMob))
-    pars <- stats::.lm.fit(DM, residuals)$coefficients
-    sigmaHat <- sd(residuals - pars[1] - pars[2]*cos(nonlinearMob) - pars[3]*sin(nonlinearMob))
-    slack <- 0.1*diff(range(fittedFMM@data))
-    Ai <- fittedFMM@A[i]
-    if(pars[1]+Ai > max(residuals)+2*sigmaHat+slack |
-       pars[1]-Ai < min(residuals)-2*sigmaHat-slack) amplitudeFlag = TRUE
-    residuals <- residuals - pars[1] - pars[2]*cos(nonlinearMob) - pars[3]*sin(nonlinearMob)
+  slack <- 0.1*diff(range(fittedFMM@data))
+  if(any(getOmega(fittedFMM) <= 0.1 &
+         2*getA(fittedFMM) > diff(range(fittedFMM@data)) + 4*sd(fittedFMM@data-fittedFMM@fittedValues) + 2*slack
+         )){
+    warning("Unusually large amplitudes detected. This may indicate issues with model parameters or input data. If omega values are too low or the dataset is sparse, consider increasing omegaMin or interpolating your data for better stability.")
   }
-  if(amplitudeFlag) warning("Detected unusually large amplitudes. Check your model parameters and input data.")
+  # if(any(getOmega(fittedFMM) <= 0.1)){
+  #
+  #   residuals <- fittedFMM@data
+  #   amplitudeFlag <- FALSE
+  #   for (i in 1:length(fittedFMM@alpha)) {
+  #     nonlinearMob = 2*atan(fittedFMM@omega[i]*tan((fittedFMM@timePoints-fittedFMM@alpha[i])/2))
+  #     DM <- cbind(fittedFMM@timePoints*0+1, cos(nonlinearMob), sin(nonlinearMob))
+  #     pars <- stats::.lm.fit(DM, residuals)$coefficients
+  #     sigmaHat <- sd(residuals - pars[1] - pars[2]*cos(nonlinearMob) - pars[3]*sin(nonlinearMob))
+  #     Ai <- fittedFMM@A[i]
+  #     if((pars[1]+Ai > max(residuals)+2*sigmaHat+slack |
+  #        pars[1]-Ai < min(residuals)-2*sigmaHat-slack) &
+  #        fittedFMM@omega[i] <= 0.1) amplitudeFlag = TRUE
+  #     residuals <- residuals - pars[1] - pars[2]*cos(nonlinearMob) - pars[3]*sin(nonlinearMob)
+  #   }
+  #   if(amplitudeFlag) warning("Unusually large amplitudes detected. This may indicate issues with model parameters or input data. If omega values are too low and the dataset is sparse, consider increasing omegaMin or interpolating your data for better stability.")
+  # }
 }
